/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "sandboxscene.h"

void renderTest(renderlist_t *list, renderpass_t *pass, engine_t *engine, int32_t i) {
  sandboxscene_t *scene;
  scene = (sandboxscene_t *)list->user;

  shaderUsePosition(pass->shader, 0, 0, 0, engine->time.current, engine->time.current, 0);
  shaderUseTexture(pass->shader, &scene->texture);
  primitiveDraw(&scene->cube, 0, -1);
}

bool sandboxSceneInit(sandboxscene_t *game) {
  int32_t passIndex;
  renderpass_t *pass;

  // Load assets
  assetTextureLoad(&game->texture, "test_texture.png");
  assetShaderLoad(&game->shader, "shaders/textured.vert", "shaders/textured.frag");
  cubeInit(&game->cube, 1.0f, 1.0f, 1.0f);

  // Initialize list
  renderListInit(&game->list, 1, 400, 400);
  game->list.user = game;

  // Set up the pass
  passIndex = renderPassAdd(&game->list);
  pass = renderListGetPass(&game->list, passIndex);
  pass->shader = &game->shader;

  return true;
}

void sandboxSceneUpdate(sandboxscene_t *game, engine_t *engine) {
  dynarray_t items;
  renderitem_t *item;
  dynArrayInit(&items, sizeof(renderitem_t), 1);
  item = (renderitem_t *)dynArrayGet(&items, dynArrayAdd(&items));
  item->onRender=  &renderTest;

  cameraLookAt(&game->camera, 3,3,3, 0,0,0);
  cameraPerspective(&game->camera, 75, 16.0f/9.0f, 0.3f, 100.0f);
  
  renderListRenderPass(&game->list, engine, &game->camera, 0, &items);
  renderListRender(&game->list, engine, &game->shader);

  dynArrayDispose(&items);
}

void sandboxSceneDispose(sandboxscene_t *game) {

}